from .interfaces import DtoGenYaml, DtoGenArgs
from .transformers import (
    Transformer,
    TransformerJava,
    TransformerPython,
    TransformerTypescript,
)

from .filewriters import (
    FileWriter,
    FileWriterJava,
    FileWriterPython,
    FileWriterTypescript,
)

import yaml
from mapperr import to_obj
from typing import Dict


class __LangClass:
    transformer: Transformer.__class__
    filewriter: FileWriter.__class__

    def __init__(
        self, transformer: Transformer.__class__, filewriter: FileWriter.__class__
    ):
        self.transformer = transformer
        self.filewriter = filewriter


_LANGUAGE_CLASSES: Dict[str, __LangClass] = {
    "java": __LangClass(TransformerJava, FileWriterJava),
    "python": __LangClass(TransformerPython, FileWriterPython),
    "typescript": __LangClass(TransformerTypescript, FileWriterTypescript),
}


class DtoGenerator:
    def __init__(self, args: DtoGenArgs):
        self.args = args

    def generate(self):
        with open(self.args.input_file) as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            data: DtoGenYaml = to_obj(data, DtoGenYaml)

        transformer: Transformer = _LANGUAGE_CLASSES[self.args.lang].transformer(
            self.args
        )

        class_info_list = []
        for class_name, dto in data.dtos.items():
            class_info = transformer.transform(class_name, dto)
            class_info_list.append(class_info)

        file_writer: FileWriter = _LANGUAGE_CLASSES[self.args.lang].filewriter(
            self.args, class_info_list, data.info
        )
        file_writer.write()



from ..interfaces import Dto, Attribute, DtoGenArgs, ClassInfo

PRIMITIVE_TYPES = {"integer", "float", "string", "boolean"}


class Transformer:
    def __init__(self, args: DtoGenArgs) -> None:
        self.args = args

    def get_class_header(self, class_name: str) -> str:
        raise NotImplementedError("get_class_header not implemented")

    def get_class_footer(self) -> str:
        raise NotImplementedError("get_class_footer not implemented")

    def get_class_name(self, class_name: str) -> str:
        raise NotImplementedError("get_class_name not implemented")

    def get_primitive_type(self, item: dict) -> str:
        raise NotImplementedError("get_primitive_attribute_line not implemented")

    def get_array_type(self) -> str:
        raise NotImplementedError("get_array_attribute_line not implemented")

    def get_map_type(self) -> str:
        raise NotImplementedError("get_map_attribute_line not implemented")

    def get_attribute_line(self, attribute_name: str, attribute_type: str) -> str:
        raise NotImplementedError("get_attribute_line not implemented")

    def transform_type(self, item: Attribute) -> str:
        raise NotImplementedError("transform_type not implemented")

    def transform(self, class_name: str, data: Dto) -> ClassInfo:
        raise NotImplementedError("transform not implemented")


class __Transformer(Transformer):
    def __init__(self, args: DtoGenArgs) -> None:
        self.args = args

    def get_class_header(self, class_name: str) -> str:
        raise NotImplementedError("get_class_header not implemented")

    def get_class_footer(self) -> str:
        raise NotImplementedError("get_class_footer not implemented")

    def get_primitive_type(self, item: dict) -> str:
        raise NotImplementedError("get_primitive_attribute_line not implemented")

    def get_array_type(self) -> str:
        raise NotImplementedError("get_array_attribute_line not implemented")

    def get_map_type(self) -> str:
        raise NotImplementedError("get_map_attribute_line not implemented")

    def get_attribute_line(self, attribute_name: str, attribute_type: str) -> str:
        raise NotImplementedError("get_attribute_line not implemented")

    def get_class_name(self, class_name: str) -> str:
        if len(class_name) < 3:
            raise ValueError("Class name must be at least 5 characters long")

        i = 1
        while i < len(class_name):
            if (class_name[i].isupper() and not class_name[i - 1].isupper()) or (
                class_name[i].islower() and not class_name[i - 1].islower()
            ):
                if not class_name[i - 1].isalpha():
                    class_name = class_name[: i - 1] + "-" + class_name[i:]
                else:
                    class_name = class_name[:i] + "-" + class_name[i:]
                i += 2
            i += 1

        # split class name with non-alphanumeric characters
        class_name_alphanum = class_name.replace("[^\w^\d]+", "-")
        return "".join(
            [
                word[0].upper() + word[1:].lower()
                for word in class_name_alphanum.split("-")
            ]
        )

    def transform_type(self, item: Attribute) -> str:
        type_ = None
        if item.type in PRIMITIVE_TYPES:
            type_ = self.get_primitive_type(item)
        elif item.type == "array":
            type_ = self.get_array_type()
            array_item_type = self.transform_type(item.items)
            type_ = type_.replace("<item>", array_item_type)
        elif item.type == "map":
            type_ = self.get_map_type()
            key_item_type = self.transform_type(item.keys)
            value_item_type = self.transform_type(item.values)
            type_ = type_.replace("<key>", key_item_type)
            type_ = type_.replace("<value>", value_item_type)
        else:
            type_ = self.get_class_name(item.type)
        return type_

    def transform(self, class_name: str, data: Dto) -> ClassInfo:
        class_text = self.get_class_header(class_name) + "\n"
        for attribute in data.attributes:
            attribute_name = attribute.name
            attribute_type = self.transform_type(attribute)
            class_text += self.get_attribute_line(attribute_name, attribute_type) + "\n"
        class_text += self.get_class_footer()

        class_info = ClassInfo()
        class_info.class_name = self.get_class_name(class_name)
        class_info.class_text = class_text

        return class_info



from typing import List
from ..interfaces import FileInfo
from .filewriter import __FileWriter


class FileWriterPython(__FileWriter):
    def get_extension(self) -> str:
        return "py"

    def generate_comment(self, comment: str) -> str:
        return f"# {comment}"

    def generate_files(self) -> List[FileInfo]:
        file_text = "from typing import List, Dict\n\n\n"
        for a_class in self.classes:
            file_text += a_class.class_text

        a_file = FileInfo()
        a_file.file_name = "dtos"
        a_file.file_text = file_text

        return [a_file]



import shutil

from unittest import TestCase
from dtogen.dtogenerator import DtoGenerator
from dtogen.interfaces import DtoGenArgs


CONTRACT_FILE_PATH = "test/dto_contract.yaml"


class TestDtoGenerator(TestCase):
    def tearDown(self) -> None:
        shutil.rmtree("out_dir")

    def test_java_generator(self):
        dto_gen_args = DtoGenArgs()
        dto_gen_args.input_file = CONTRACT_FILE_PATH
        dto_gen_args.output_dir = "out_dir"
        dto_gen_args.lang = "java"
        dto_gen_args.java_package = "com.example.hello"

        generator = DtoGenerator(dto_gen_args)
        generator.generate()

    def test_python_generator(self):
        dto_gen_args = DtoGenArgs()
        dto_gen_args.input_file = CONTRACT_FILE_PATH
        dto_gen_args.output_dir = "out_dir"
        dto_gen_args.lang = "python"

        generator = DtoGenerator(dto_gen_args)
        generator.generate()

    def test_typescript_generator(self):
        dto_gen_args = DtoGenArgs()
        dto_gen_args.input_file = CONTRACT_FILE_PATH
        dto_gen_args.output_dir = "out_dir"
        dto_gen_args.lang = "typescript"

        generator = DtoGenerator(dto_gen_args)
        generator.generate()



import argparse
from dtogen.interfaces import DtoGenArgs
from .dtogenerator import DtoGenerator

def main():
    arg_parser = argparse.ArgumentParser(description="Generate DTOs from a YAML file")
    arg_parser.add_argument(
        "-i", "--input", metavar="INPUT", type=str, required=True, help="input YAML file"
    )
    arg_parser.add_argument(
        "-o", "--output", metavar="OUTPUT", type=str, required=True, help="output directory"
    )
    arg_parser.add_argument(
        "-l",
        "--lang",
        metavar="LANG",
        type=str,
        required=True,
        help="output language",
        choices=["python", "java", "typescript"],
    )
    arg_parser.add_argument(
        "--java-package", metavar="PACKAGE", type=str, help="Java package"
    )

    args = arg_parser.parse_args()

    if args.lang == "java" and not args.java_package:
        arg_parser.error("--java-package is required for java output")

    dtogen_args = DtoGenArgs()
    dtogen_args.input_file = args.input
    dtogen_args.lang = args.lang
    dtogen_args.output_dir = args.output
    dtogen_args.java_package = args.java_package

    dto_generator = DtoGenerator(dtogen_args)

    dto_generator.generate()



from typing import Dict, List


class Info:
    name: str
    version: str


class NestedAttribute:
    type: str
    items: "NestedAttribute"
    keys: "NestedAttribute"
    values: "NestedAttribute"


class Attribute(NestedAttribute):
    name: str


class Dto:
    attributes: List[Attribute]


class DtoGenYaml:
    info: Info
    dtos: Dict[str, Dto]


class ClassInfo:
    class_name: str
    class_text: str


class FileInfo:
    file_name: str
    file_text: str


class DtoGenArgs:
    input_file: str
    output_dir: str
    lang: str
    java_package: str



from .transformer import __Transformer
from ..interfaces import NestedAttribute


class TransformerJava(__Transformer):
    def get_class_header(self, class_name: str) -> str:
        header = f"package {self.args.java_package};\n\n"
        header += f"public class {self.get_class_name(class_name)} {'{'}"
        return header

    def get_class_footer(self) -> str:
        return "}\n"

    def get_primitive_type(self, item: NestedAttribute) -> str:
        return {
            "integer": "Integer",
            "float": "Float",
            "string": "String",
            "boolean": "Boolean",
        }[item.type]

    def get_array_type(self) -> str:
        return "List<<item>>"

    def get_map_type(self) -> str:
        return "Map<<key>, <value>>"

    def get_attribute_line(self, attribute_name: str, attribute_type: str) -> str:
        return f"    public {attribute_type} {attribute_name};"



from .transformer import __Transformer
from ..interfaces import NestedAttribute


class TransformerTypescript(__Transformer):
    def get_class_header(self, class_name: str) -> str:
        return f"export interface {self.get_class_name(class_name)} {'{'}"

    def get_class_footer(self) -> str:
        return "}\n\n\n"

    def get_primitive_type(self, item: NestedAttribute) -> str:
        return {
            "integer": "number",
            "float": "number",
            "string": "string",
            "boolean": "boolean",
        }[item.type]

    def get_array_type(self) -> str:
        return "<item>[]"

    def get_map_type(self) -> str:
        return "Record<<key>, <value>>"

    def get_attribute_line(self, attribute_name: str, attribute_type: str) -> str:
        return f"    {attribute_name}: {attribute_type}"



from typing import List
from ..interfaces import FileInfo
from .filewriter import __FileWriter


class FileWriterJava(__FileWriter):
    def get_extension(self) -> str:
        return "java"

    def generate_comment(self, comment: str) -> str:
        return f"// {comment}"

    def generate_files(self) -> List[FileInfo]:
        files = []
        for a_class in self.classes:
            a_file = FileInfo()
            a_file.file_name = a_class.class_name
            a_file.file_text = a_class.class_text
            files.append(a_file)
        return files



import os

from typing import List
from ..interfaces import ClassInfo, FileInfo, DtoGenArgs, Info


class FileWriter:
    args: DtoGenArgs
    classes: List[ClassInfo]
    dto_gen_info: Info

    def __init__(self, args: DtoGenArgs, classes: List[ClassInfo], dto_gen_info: Info):
        self.args = args
        self.classes = classes
        self.dto_gen_info = dto_gen_info

    def get_extension(self) -> str:
        raise NotImplementedError("get_extension not implemented")

    def generate_files(self) -> List[FileInfo]:
        raise NotImplementedError("generate_files not implemented")

    def generate_comment(self, comment: str) -> str:
        raise NotImplementedError("generate_comment not implemented")

    def write(self):
        raise NotImplementedError("write not implemented")


class __FileWriter(FileWriter):
    def get_extension(self) -> str:
        raise NotImplementedError("get_extension not implemented")

    def generate_files(self) -> List[FileInfo]:
        raise NotImplementedError("generate_files not implemented")

    def generate_comment(self, comment: str) -> str:
        raise NotImplementedError("generate_comment not implemented")

    def write(self):
        if os.path.exists(self.args.output_dir):
            raise Exception("folder already exists")

        os.mkdir(self.args.output_dir)

        files_to_write = self.generate_files()
        for file in files_to_write:
            comment_lines = "\n\n" + "\n".join(
                [
                    self.generate_comment("DO NOT MAKE CHANGES ON THIS FILE"),
                    self.generate_comment("IT IS AUTO GENERATED BY dtogen"),
                    self.generate_comment(
                        f"source > {self.dto_gen_info.name}:{self.dto_gen_info.version}"
                    ),
                ]
            )
            with open(
                f"{self.args.output_dir}/{file.file_name}.{self.get_extension()}", "w"
            ) as f:
                f.write(file.file_text + comment_lines)



from unittest import TestCase
from dtogen.transformers.transformer_java import TransformerJava


class TestTransform(TestCase):
    def test_class_name_converter_when_class_name_is_all_lowercase(self):
        transformer = TransformerJava(None)
        class_name = "test"
        expected = "Test"
        actual = transformer.get_class_name(class_name)
        self.assertEqual(expected, actual)

    def test_class_name_converter_when_class_name_is_all_uppercase(self):
        transformer = TransformerJava(None)
        class_name = "TEST"
        expected = "Test"
        actual = transformer.get_class_name(class_name)
        self.assertEqual(expected, actual)

    def test_class_name_converter_when_class_name_is_mixed_case(self):
        transformer = TransformerJava(None)
        class_name = "testClass"
        expected = "TestClass"
        actual = transformer.get_class_name(class_name)
        self.assertEqual(expected, actual)

    def test_class_name_converter_when_class_name_is_mixed_case_with_non_alphanumeric_characters(
        self,
    ):
        transformer = TransformerJava(None)
        class_name = "test-class-123"
        expected = "TestClass123"
        actual = transformer.get_class_name(class_name)
        self.assertEqual(expected, actual)



import os.path
from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="dtogen",
    version="0.0.10",
    description="DTO generator for Java, TypeScript, Python",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/mujdecisy/dtogen",
    keywords=["python", "DTO", "generator"],
    author="mujdecisy",
    author_email="mujdecisy@gmail.com",
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    packages=['dtogen','dtogen.transformers','dtogen.filewriters'],
    include_package_data=True,
    install_requires=["mapperr==0.2.4", "pyyaml"],
)
